"""
Module to manage CDE Clusters
"""

from cdepy.cdeconnection import CdeConnection
from datetime import datetime
import pytz
from os.path import exists
from requests_toolbelt import MultipartEncoder
import xmltodict as xd
import pyparsing
import os, json, requests, re, sys


class CdeClusterManager:
    """
    Class to manage CDE Clusters
    """

    def __init__(self, cdeConnection):
        self.clusterConnection = cdeConnection
        self.JOBS_API_URL = self.clusterConnection.JOBS_API_URL
        self.TOKEN = self.clusterConnection.TOKEN

    def createJob(self, cdeJobDefinition):
        """
        Method to create a CDE Job
        Requires cdeJobDefinition of type cdeAirflowJobDefinition or cdeSparkJobDefinition as input for payload
        """

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        PUT = '{}/jobs'.format(self.JOBS_API_URL)

        data = json.dumps(cdeJobDefinition)

        x = requests.post(PUT, headers=headers, data=data)

        if x.status_code == 201:
            print("CDE Job Creation Succeeded\n")
        else:
            print(x.status_code)
            print(x.text)


    def deleteJob(self):
        raise NotImplementedError

    def listJobs(self):
        raise NotImplementedError

    def listJobRuns(self):
        """
        Method to show all CDE Jobs that have been executed in the cluster
        Does not require input
        """

        tz_LA = pytz.timezone('America/Los_Angeles')
        now = datetime.now(tz_LA)
        print("Listing Jobs as of: {} PACIFIC STANDARD TIME\n".format(now))

        url = self.JOBS_API_URL + "/job-runs"

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.get(url+'?limit=100&offset=0&orderby=ID&orderasc=false', headers=headers)

        return x.text

        if x.status_code == 201:
            print("Listing Jobs Succeeded")
        else:
            print(x.status_code)
            print(x.text)

    def runJob(self, CDE_JOB_NAME):
        """
        Method to trigger execution of CDE Job
        CDE Job could be of type Spark or Airflow
        The method assumes the CDE Job has already been created
        """

        payloadData = {"hidden":False}

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        POST = "{}/jobs/".format(self.JOBS_API_URL)+CDE_JOB_NAME+"/run"

        data = json.dumps(payloadData)

        x = requests.post(POST, headers=headers, data=data)

        if x.status_code == 201:
            print("CDE Job Submission has Succeeded\n")
            print("Please visit the CDE Job Runs UI to validate CDE Job Status\n")
        else:
            print(x.status_code)
            print(x.text)

    def createResource(self, cdeRsourceDefinition):
        """
        Method to create CDE Resource
        Requires cdeRsourceDefinition as input
        Accepts types Files or Python
        e.g. cdeRsourceDefinition = {"name": str(resource_name)}
        """

        print("CDE Resource Creation in Progress\n")

        url = self.JOBS_API_URL + "/resources"
        data_to_send = json.dumps(cdeRsourceDefinition).encode("utf-8")

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.post(url, data=data_to_send, headers=headers)

        if x.status_code == 201:
            print("CDE Resource Created Successfully")
        else:
            print(x.status_code)
            print(x.text)

    def deleteResource(self, cdeResourceName):
        """
        Method to delete CDE Resource
        Requires cdeRsourceDefinition name as input
        e.g. cdeResourceName = str(resource_name)
        """

        print("CDE Resource Deletion in Progress\n")

        headers = {
            'accept': 'application/json',
        }

        url = self.JOBS_API_URL + "/resources/" + cdeResourceName

        response = requests.delete(url, headers=headers)

        if x.status_code == 201:
            print("CDE Resource Deleted Successfully\n")
        else:
            print(x.status_code)
            print(x.text)

    #Upload Spark CDE Job file to CDE Resource
    def uploadFile(self, CDE_RESOURCE_NAME, LOCAL_FILE_PATH, LOCAL_FILE_NAME):
        """
        Method to uplaod files from local to CDE Resource
        Can be used to:
            1) upload files to a CDE Resource of type Files
            2) uplaod a "requirements.txt" file to a CDE Resource of type Python Environment
        Requires a CDE_RESOURCE_NAME, LOCAL_FILE_PATH and LOCAL_FILE_NAME
        e.g. "myCdeFilesResource", "~/myfiles/cdefiles", and "mySparkScript.py"
        e.g. "myCdePythonResource", "~/myfiles/cdefiles", and "requirements.txt"
        """

        print("Uploading File {0} to CDE Resource {1}\n".format(LOCAL_FILE_NAME, CDE_RESOURCE_NAME))

        m = MultipartEncoder(
            fields={
                    'file': ('filename', open(LOCAL_FILE_PATH+"/"+LOCAL_FILE_NAME, 'rb'), 'text/plain')}
            )

        PUT = '{jobs_api_url}/resources/{resource_name}/{file_name}'.format(jobs_api_url=self.JOBS_API_URL, resource_name=CDE_RESOURCE_NAME, file_name=LOCAL_FILE_NAME)

        x = requests.put(PUT, data=m, headers={'Authorization': f"Bearer {self.TOKEN}",'Content-Type': m.content_type})
        print("Response Status Code {}".format(x.status_code))

        if x.status_code == 201:
            print("Uploading File {0} to CDE Resource {1} has Succeeded\n".format(LOCAL_FILE_NAME, CDE_RESOURCE_NAME))
        else:
            print(x.status_code)
            print(x.text)



"""
Module to define CDE Jobs
"""

from abc import ABC, abstractmethod
from cdepy.cdeconnection import CdeConnection

class CdeJob(ABC):
    """
    Class to define CDE Job
    """
    @abstractmethod
    def createJobDefinition(self):
        pass


class CdeSparkJob(CdeJob):
    """
    Class to define CDE Spark Jobs
    """

    def __init__(self, cdeConnection):
        self.clusterConnection = cdeConnection
        self.WORKLOAD_USER = self.clusterConnection.WORKLOAD_USER

    def createJobDefinition(self, CDE_JOB_NAME, CDE_RESOURCE_NAME, APPLICATION_FILE_NAME, SPARK_CONFS={"spark.pyspark.python": "python3"}):
        """
        Method to create CDE Spark Job Definition
        Requires CDE Job Name, CDE Files Resource Name, Application File Name, and optionally spark configs
        """

        ### Any Spark Job Configuration Options (Not Mandatory) ###
        #spark_confs_example = {
                  #"spark.dynamicAllocation.maxExecutors": "6",
                  #"spark.dynamicAllocation.minExecutors": "2",
                  #"spark.executor.extraJavaOptions": "-Dsun.security.krb5.debug=true -Dsun.security.spnego.debug=true",
                  #"spark.hadoop.fs.s3a.metadatastore.impl": "org.apache.hadoop.fs.s3a.s3guard.NullMetadataStore",
                  #"spark.kubernetes.memoryOverheadFactor": "0.2",
                  #"spark.pyspark.python": "python3"
                  #"spark.rpc.askTimeout": "600",
                  #"spark.sql.shuffle.partitions": "48",
                  #"spark.yarn.access.hadoopFileSystems": "s3a://your_data_lake_here"
                #}

        cdeSparkJobDefinition = {
              "name": CDE_JOB_NAME,# CDE Job Name As you want it to appear in the CDE JOBS UI
              "type": "spark",
              "retentionPolicy": "keep_indefinitely",
              "mounts": [
                {
                  "resourceName": CDE_RESOURCE_NAME
                }
              ],
              "spark": {
                "file": APPLICATION_FILE_NAME,
                "driverMemory": "1g",
                "driverCores": 1, #this must be an integer
                "executorMemory": "4g",
                "executorCores": 1, #this must be an integer
                "conf": SPARK_CONFS,
                "logLevel": "INFO"
              },
              "schedule": {
                "enabled": False,
                "user": self.WORKLOAD_USER #Your CDP Workload User is automatically set by CML as an Environment Variable
              }
            }

        return cdeSparkJobDefinition


class CdeAirflowJob(CdeJob):
    """
    Class to define CDE Airflow Jobs
    """

    def __init__(self, cdeConnection):
        self.clusterConnection = cdeConnection
        self.WORKLOAD_USER = self.clusterConnection.WORKLOAD_USER

    def createJobDefinition(self, CDE_JOB_NAME, DAG_FILE):
        """
        Method to create CDE Job Definition of type Airflow
        Requires CDE Job Name, Application File Name and optionally CDE Files Resource Name
        """

        cdeAirflowJobDefinition = {
              "name": CDE_JOB_NAME,# CDE Job Name As you want it to appear in the CDE JOBS UI
              "type": "airflow",
              "retentionPolicy": "keep_indefinitely",
              "dagFile": DAG_FILE
            }

        return cdeAirflowJobDefinition



from setuptools import setup, find_packages

setup(
    author="Paul de Fusco",
    description="A Python Package for interacting with Cloudera Data Engineering Clusters",
    long_description="A Python Package for interacting with Cloudera Data Engineering Clusters",
    name="cdepy",
    version="0.1.1",
    packages=find_packages(include=["cdepy", "cdepy.*"]),
    install_requires=["pyparsing==3.0.9", "requests-toolbelt==1.0.0", "xmltodict==0.13.0"],
    python_requires=">=2.7",
    readme="README.md"
)



"""
Helper Methods
"""

import requests
import json

def listVcMeta(self, token):

    headers = {
        'Authorization': f"Bearer {token}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
        }

    x = requests.get(self.JOBS_API_URL+'/info', headers=headers)
    cde_vc_name = json.loads(x.text)["appName"]
    #cde_vc_id = json.loads(x.text)["appId"]
    #cde_vc_console_url = json.loads(x.text)["cdeConsoleURL"]
    #cde_cluster_id = json.loads(x.text)["clusterID"]
    #cde_version = json.loads(x.text)["version"]

    return cdeVcName

def runSparkJob(self, token, cde_job_name, driver_cores = 2, driver_memory = "4g", executor_cores = 4, executor_memory = "4g", num_executors = 4):

    print("Started to Submit Spark Job {}".format(cde_job_name))

    cde_payload = {"overrides":
                   {"spark":
                    {"driverCores": driver_cores,
                     "driverMemory": driver_memory,
                     "executorCores": executor_cores,
                     "executorMemory": executor_memory,
                     "numExecutors": num_executors}
                   }
                  }

    headers = {
        'Authorization': f"Bearer {token}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    POST = "{}/jobs/".format(self.JOBS_API_URL)+cde_job_name+"/run"

    data = json.dumps(cde_payload)

    x = requests.post(POST, headers=headers, data=data)

    if x.status_code == 201:
        print("Submitting CDE Spark Job {} has Succeeded".format(cde_job_name))
        print("This doesn't necessarily mean that the CDE Spark Job has Succeeded")
        print("Please visit the CDE Job Runs UI to check on CDE Job Status")
    else:
        print(x.status_code)
        print(x.text)
